// setdisplay Copyright (c) 2014 Elmar de Koning edekoning@gmail.com

#include <iostream>
#include <Windows.h>
#include <Winuser.h>

void ListDisplayDevices ()
{
    DISPLAY_DEVICE adapter;
    adapter.cb = sizeof (DISPLAY_DEVICE);
    int adapNum= 0;

    while (EnumDisplayDevices (NULL, adapNum, &adapter, 0))
    {
        std::wcout << '\n'
                   << adapter.DeviceName << ", "
                   << adapter.DeviceString << '\n';
        if (adapter.StateFlags)
        {
            std::wcout << "  state   : "
                       << (adapter.StateFlags & DISPLAY_DEVICE_ACTIVE ? "active, " : "")
                       << (adapter.StateFlags & DISPLAY_DEVICE_MIRRORING_DRIVER ? "mirroring, " : "")
                       << (adapter.StateFlags & DISPLAY_DEVICE_MODESPRUNED ? "pruned, " : "")
                       << (adapter.StateFlags & DISPLAY_DEVICE_PRIMARY_DEVICE ? "primary, " : "")
                       << (adapter.StateFlags & DISPLAY_DEVICE_REMOVABLE ? "removable, " : "")
                       << (adapter.StateFlags & DISPLAY_DEVICE_VGA_COMPATIBLE ? "vga, " : "")
                       << '\n';
        }

        DEVMODE mode;
        mode.dmSize = sizeof (DEVMODE);

        if (EnumDisplaySettings (adapter.DeviceName, ENUM_REGISTRY_SETTINGS, &mode) && mode.dmPelsWidth)
        {
            std::cout << "  settings: ";
            std::cout << mode.dmPelsWidth << " x "
                      << mode.dmPelsHeight << " @ "
                      << mode.dmDisplayFrequency << "Hz - "
                      << mode.dmBitsPerPel << "bpp\n";
            std::cout << "  position: ";
            std::cout << mode.dmPosition.x << ", "
                      << mode.dmPosition.y << "\n";
        }

        DISPLAY_DEVICE monitor;
        monitor.cb = sizeof (DISPLAY_DEVICE);
        int monNum = 0;

        while (EnumDisplayDevices (adapter.DeviceName, monNum, &monitor, 1)) // EDD_GET_DEVICE_INTERFACE_NAME
        {
            std::wcout << "  monitor : ";
            std::wcout << monitor.DeviceName << ", "
                       << monitor.DeviceString << '\n';
            ++monNum;
        }
        ++adapNum;
    }
    std::cout << std::endl;
}

bool GetDisplayDevice (DISPLAY_DEVICE *displayDevice, DEVMODE *devMode, const std::string& target="")
{
    int index = 0;
    while (EnumDisplayDevices (NULL, index++, displayDevice, 0))
    {
        //  retrieve the target display device, or the primary one if no target was specified
        if ((target.empty () && displayDevice->StateFlags & DISPLAY_DEVICE_PRIMARY_DEVICE) ||
            (std::wstring (target.begin (), target.end ()) == displayDevice->DeviceName))
        {
            if (EnumDisplaySettings (displayDevice->DeviceName, ENUM_REGISTRY_SETTINGS, devMode))
            {
                return true;
            }
        }
    }
    return false;
}

void SwitchPrimaryDisplayDevice (const std::string& target)
{
    DISPLAY_DEVICE primaryDev;
    DEVMODE primaryMode;
    primaryDev.cb = sizeof (DISPLAY_DEVICE);
    primaryMode.dmSize = sizeof (DEVMODE);

    DISPLAY_DEVICE targetDev;
    DEVMODE targetMode;
    targetDev.cb = sizeof (DISPLAY_DEVICE);
    targetMode.dmSize = sizeof (DEVMODE);

    // get the primary and target display devices
    if (GetDisplayDevice (&primaryDev, &primaryMode) &&
        GetDisplayDevice (&targetDev, &targetMode, target))
    {
        // make sure the target display device is not the primary one
        if (wcscmp (primaryDev.DeviceName, targetDev.DeviceName))
        {
            std::wcout << "switching primary display device:\n"
                       << "  from " << primaryDev.DeviceName << ", " << primaryDev.DeviceString << '\n'
                       << "  to   " << targetDev.DeviceName << ", " << targetDev.DeviceString << '\n';

            // change primary display device
            primaryMode.dmFields = DM_POSITION;
            primaryMode.dmPosition.x =
                0 < targetMode.dmPosition.x
                ? - primaryMode.dmPelsWidth
                : + targetMode.dmPelsWidth;
            primaryMode.dmPosition.y = 0;
            ChangeDisplaySettingsEx (primaryDev.DeviceName, &primaryMode, NULL, CDS_NORESET | CDS_UPDATEREGISTRY, NULL);
            // change target display device
            targetMode.dmFields = DM_POSITION;
            targetMode.dmPosition.x = 0;
            targetMode.dmPosition.y = 0;
            ChangeDisplaySettingsEx (targetDev.DeviceName, &targetMode, NULL, CDS_NORESET | CDS_UPDATEREGISTRY | CDS_SET_PRIMARY, NULL);
            // commit display changes
            ChangeDisplaySettings (NULL, 0);
        }
    }
    else
    {
        std::wcout << "unable to find the primary and target display device\n";
    }
    std::cout << std::endl;
}

int main(int argc, char *argv[])
{
    if (argc < 2) {
		std::cout << "setdisplay Copyright (c) 2014 Elmar de Koning edekoning@gmail.com\n\n";
        std::cout << "usage: setdisplay <target> | -l \n\n";
        std::cout << "       <target>    the display device to set as primary\n";
        std::cout << "       -l          lists all display devices\n";
        return 1;
    }
    std::string arg = argv [1];
    if ("-l" == arg) {
        ListDisplayDevices ();
        return 0;
    }
    else {
        SwitchPrimaryDisplayDevice (arg);
        return 0;
    }
}
